# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import json
import logging

from grimoirelab_toolkit.datetime import str_to_datetime

from ...backend import (Backend,
                        BackendCommand,
                        BackendCommandArgumentParser)
from ...client import HttpClient
from ...utils import DEFAULT_DATETIME

MOZILLA_CLUB_URL = \
    "https://spreadsheets.google.com/feeds/cells/1QHl2bjBhMslyFzR5XXPzMLdzzx7oeSKTbgR5PM8qp64/ohaibtm/public/values?alt=json"

EVENT_TEMPLATE = {
    1: "Status",
    2: "Date of Event",
    3: "Club Name",
    4: "Country",
    5: "Your Name",
    6: "Your Twitter Handle (Optional)",
    7: "Event Description",
    8: "Attendance",
    9: "Club Link",
    10: "City",
    11: "Event Creations",
    12: "Web Literacy Skills",
    13: "Links to Curriculum (Optional)",
    14: "Links to Blogpost (Optional)",
    15: "Links to Video (Optional)",
    16: "Links to Photos (Optional)",
    17: "Feedback from Attendees",
    18: "Your Feedback",
    19: "Timestamp",
    20: "Event Cover Photo"
}

CATEGORY_EVENT = 'event'

logger = logging.getLogger(__name__)


class MozillaClub(Backend):
    """MozillaClub backend for Perceval.

    This class retrieves the data from MozillaClub.

    :param url: Mozilla Club Events url
    :param tag: label used to mark the data
    :param archive: archive to store/retrieve items
    :param ssl_verify: enable/disable SSL verification
    """
    version = '0.5.0'

    CATEGORIES = [CATEGORY_EVENT]
    EXTRA_SEARCH_FIELDS = {
        'club_name': ['Club Name']
    }

    def __init__(self, url=MOZILLA_CLUB_URL, tag=None, archive=None, ssl_verify=True):
        origin = url
        self.url = url
        super().__init__(origin, tag=tag, archive=archive, ssl_verify=ssl_verify)

        self.client = None

    def fetch(self, category=CATEGORY_EVENT):
        """Fetch events from the MozillaClub URL.

        The method retrieves, from a MozillaClub URL, the
        events. The data is a Google spreadsheet retrieved using
        the feed API REST.

        :param category: the category of items to fetch

        :returns: a generator of events
        """
        kwargs = {}
        items = super().fetch(category, **kwargs)

        return items

    def fetch_items(self, category, **kwargs):
        """Fetch events

        :param category: the category of items to fetch
        :param kwargs: backend arguments

        :returns: a generator of items
        """
        logger.info("Looking for events at url '%s'", self.url)

        nevents = 0  # number of events processed

        raw_cells = self.client.get_cells()
        parser = MozillaClubParser(raw_cells)

        for event in parser.parse():
            yield event
            nevents += 1

        logger.info("Total number of events: %i", nevents)

    @classmethod
    def has_archiving(cls):
        """Returns whether it supports archiving items on the fetch process.

        :returns: this backend supports items archive
        """
        return True

    @classmethod
    def has_resuming(cls):
        """Returns whether it supports to resume the fetch process.

        :returns: this backend supports items resuming
        """
        return False

    @staticmethod
    def metadata_id(item):
        """Extracts the identifier from an event item."""

        return str(item['Date of Event'] + "_" + item['Club Name'])

    @staticmethod
    def metadata_category(item):
        """Extracts the category from a item.

        This backend only generates one type of item which is
        'event'.
        """
        return CATEGORY_EVENT

    @staticmethod
    def metadata_updated_on(item):
        """Extracts the update time from a MozillaClub item.

        The timestamp is extracted from 'updated' field.
        This date is in ISO format and it needs to be converted to
        a float value.

        :param item: item generated by the backend

        :returns: a UNIX timestamp
        """
        date = str_to_datetime(item['updated'])
        return float(date.timestamp())

    def _init_client(self, from_archive=False):
        """Init client"""

        return MozillaClubClient(self.url, self.archive, from_archive, self.ssl_verify)


class MozillaClubClient(HttpClient):
    """MozillaClub API client.

    This class implements a simple client to retrieve events from
    projects in a MozillaClub site.

    :param url: URL of MozillaClub
    :param archive: an archive to store/read fetched data
    :param from_archive: it tells whether to write/read the archive
    :param ssl_verify: enable/disable SSL verification

    :raises HTTPError: when an error occurs doing the request
    """
    def __init__(self, url, archive=None, from_archive=False, ssl_verify=True):
        super().__init__(url, archive=archive, from_archive=from_archive, ssl_verify=ssl_verify)

    def get_cells(self):
        """Retrieve all cells from the spreadsheet."""

        logger.info("Retrieving all cells spreadsheet data ...")
        logger.debug("MozillaClub client calls API: %s", self.base_url)
        raw_cells = self.fetch(self.base_url)

        return raw_cells.text


class MozillaClubParser:
    """Git log parser.

    This class parses a string in JSON format from a Google Spreadsheet. The
    feed includes all the cells in a list.

    Events are rows in the spreadsheet. The columns are the fields
    for the event. The JSON retrieved from the spreadsheet feed is
    a plain list with all the cells from all the rows.

    The list of cells is returned in the feed raw JSON in:

    {
    "encoding": "UTF-8",
    "feed": {
        "entry": [cell-1, ..., cell-n],
        "updated": {
            "$t": "2016-12-13T15:44:04.821Z"
        },
        ...
    },
    "version": "1.0"
    }

    The format of a cell is:

    {
        "category": [
            {
                "scheme": "http://schemas.google.com/spreadsheets/2006",
                "term": "http://schemas.google.com/spreadsheets/2006#cell"
            }
        ],
        "content": {
            "$t": "Status",
            "type": "text"
        },
        "gs$cell": {
            "$t": "Status",
            "col": "1",
            "row": "1"
        },
        "id": {
            "$t": "https://spreadsheets.google.com/feeds/cells/1QHl2b...
        },
        "link": [
            {
                "href": "https://spreadsheets.google.com/feeds/cells/1QHl2b...
                "rel": "self",
                "type": "application/atom+xml"
            }
        ],
        "title": {
            "$t": "A1",
            "type": "text"
        },
        "updated": {
            "$t": "2016-12-13T15:44:04.821Z"
        }
    }
    """
    def __init__(self, feed):
        self.feed = feed  # Spreadsheet feed
        self.cells = None  # list with all cells to be processed
        self.ncell = None  # current cell being parsed

    def parse(self):
        """Parse the MozillaClub spreadsheet feed cells json."""

        nevents_wrong = 0

        feed_json = json.loads(self.feed)

        if 'entry' not in feed_json['feed']:
            return

        self.cells = feed_json['feed']['entry']
        self.ncell = 0

        event_fields = self.__get_event_fields()

        # Process all events reading the rows according to the event template
        # The only way to detect the end of row is looking to the
        # number of column. When the max number is reached (cell_cols) the next
        # cell is from the next row.
        while self.ncell < len(self.cells):
            # Process the next row (event) getting all cols to build the event
            event = self.__get_next_event(event_fields)

            if event['Date of Event'] is None or event['Club Name'] is None:
                logger.warning("Wrong event data: %s", event)
                nevents_wrong += 1
                continue
            yield event

        logger.info("Total number of wrong events: %i", nevents_wrong)

    def __get_event_fields(self):
        """Get the events fields (columns) from the cells received."""

        event_fields = {}
        # The cells in the first row are the column names
        # Check that the columns names are the same we have as template
        # Create the event template from the data retrieved
        while self.ncell < len(self.cells):
            cell = self.cells[self.ncell]
            row = cell['gs$cell']['row']
            if int(row) > 1:
                # When the row number >1 the column row is finished
                break
            ncol = int(cell['gs$cell']['col'])
            name = cell['content']['$t']
            event_fields[ncol] = name
            if ncol in EVENT_TEMPLATE:
                if event_fields[ncol] != EVENT_TEMPLATE[ncol]:
                    logger.warning("Event template changed in spreadsheet %s vs %s",
                                   name, EVENT_TEMPLATE[ncol])
            else:
                logger.warning("Event template changed in spreadsheet. New column: %s", name)

            self.ncell += 1
        return event_fields

    def __get_next_event(self, event_fields):
        # Fill the empty event with all fields as None
        event = {key: None for key in event_fields.values()}
        event['updated'] = DEFAULT_DATETIME.isoformat()

        last_col = 0
        while self.ncell < len(self.cells):
            # Get all cols (cells) for the event (row)
            cell = self.cells[self.ncell]
            ncol = int(cell['gs$cell']['col'])
            if ncol <= last_col:
                # new event (row) detected: new cell column lower than last
                break
            event[event_fields[ncol]] = cell['content']['$t']
            # Add an extra column with the update datetime
            cell_update = str_to_datetime(cell['updated']['$t'])
            if cell_update > str_to_datetime(event['updated']):
                event['updated'] = cell['updated']['$t']
            last_col = ncol
            self.ncell += 1

        return event


class MozillaClubCommand(BackendCommand):
    """Class to run MozillaClub backend from the command line."""

    BACKEND = MozillaClub

    @classmethod
    def setup_cmd_parser(cls):
        """Returns the MozillaClub argument parser."""

        parser = BackendCommandArgumentParser(cls.BACKEND,
                                              archive=True,
                                              ssl_verify=True)

        # Required arguments
        parser.parser.add_argument('url', nargs='?',
                                   default=MOZILLA_CLUB_URL,
                                   help="MozillaClub URL")

        return parser



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import datetime
import json
import unittest

import httpretty

from perceval.backend import BackendCommandArgumentParser
from perceval.backends.mozilla.crates import (Crates,
                                              CratesClient,
                                              CratesCommand,
                                              SLEEP_TIME,
                                              CATEGORY_CRATES,
                                              CATEGORY_SUMMARY)
from perceval.utils import DEFAULT_DATETIME
from base import TestCaseBackendArchive

CRATES_API_URL = "https://crates.io/api/v1/"


def read_file(filename, mode='r'):
    with open(filename, mode) as f:
        content = f.read()
    return content


def setup_http_server(empty=False):
    """Setup a mock HTTP server"""

    summary = read_file('data/crates/crates_summary')
    httpretty.register_uri(httpretty.GET,
                           CRATES_API_URL + "summary",
                           body=summary,
                           status=200)

    if empty:
        page_empty = read_file('data/crates/crates_page_empty')
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates?sort=alphabetical&page=1',
                               body=page_empty,
                               status=200)
    else:
        page_1 = read_file('data/crates/crates_page_1')
        page_2 = read_file('data/crates/crates_page_2')

        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates?sort=alphabetical&page=2',
                               body=page_2,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates?sort=alphabetical&page=1',
                               body=page_1,
                               status=200)

        crate_1 = read_file('data/crates/crate_example_1')
        crate_2 = read_file('data/crates/crate_example_2')
        crate_3 = read_file('data/crates/crate_example_3')
        crate_4 = read_file('data/crates/crate_example_4')

        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/a',
                               body=crate_1,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aabb2',
                               body=crate_2,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aac',
                               body=crate_3,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/abc',
                               body=crate_4,
                               status=200)

        crate_user_1 = read_file('data/crates/crate_owner_user_1')
        crate_user_2 = read_file('data/crates/crate_owner_user_2')
        crate_user_3 = read_file('data/crates/crate_owner_user_3')
        crate_user_4 = read_file('data/crates/crate_owner_user_4')

        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/a/owner_user',
                               body=crate_user_1,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aabb2/owner_user',
                               body=crate_user_2,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aac/owner_user',
                               body=crate_user_3,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/abc/owner_user',
                               body=crate_user_4,
                               status=200)

        crate_team_1 = read_file('data/crates/crate_owner_team_1')
        crate_team_2 = read_file('data/crates/crate_owner_team_2')
        crate_team_3 = read_file('data/crates/crate_owner_team_3')
        crate_team_4 = read_file('data/crates/crate_owner_team_4')

        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/a/owner_team',
                               body=crate_team_1,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aabb2/owner_team',
                               body=crate_team_2,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aac/owner_team',
                               body=crate_team_3,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/abc/owner_team',
                               body=crate_team_4,
                               status=200)

        crate_version_downloads_1 = read_file('data/crates/crate_version_downloads_1')
        crate_version_downloads_2 = read_file('data/crates/crate_version_downloads_empty')
        crate_version_downloads_3 = read_file('data/crates/crate_version_downloads_empty')
        crate_version_downloads_4 = read_file('data/crates/crate_version_downloads_empty')

        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/a/downloads',
                               body=crate_version_downloads_1,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aabb2/downloads',
                               body=crate_version_downloads_2,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aac/downloads',
                               body=crate_version_downloads_3,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/abc/downloads',
                               body=crate_version_downloads_4,
                               status=200)

        crate_versions_1 = read_file('data/crates/crate_versions_1')
        crate_versions_2 = read_file('data/crates/crate_versions_2')
        crate_versions_3 = read_file('data/crates/crate_versions_3')
        crate_versions_4 = read_file('data/crates/crate_versions_4')

        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/a/versions',
                               body=crate_versions_1,
                               status=200)

        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aabb2/versions',
                               body=crate_versions_2,
                               status=200)

        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/aac/versions',
                               body=crate_versions_3,
                               status=200)
        httpretty.register_uri(httpretty.GET,
                               CRATES_API_URL + 'crates/abc/versions',
                               body=crate_versions_4,
                               status=200)


class TestCratesBackend(unittest.TestCase):
    """Crates.io backend tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        crates = Crates(tag='test')

        self.assertEqual(crates.origin, 'https://crates.io/')
        self.assertEqual(crates.tag, 'test')
        self.assertIsNone(crates.client)
        self.assertTrue(crates.ssl_verify)

        # When tag is empty or None it will be set to
        # the value in origin
        crates = Crates(ssl_verify=False)
        self.assertEqual(crates.origin, 'https://crates.io/')
        self.assertEqual(crates.tag, 'https://crates.io/')
        self.assertFalse(crates.ssl_verify)

        crates = Crates(tag='')
        self.assertEqual(crates.origin, 'https://crates.io/')
        self.assertEqual(crates.tag, 'https://crates.io/')
        self.assertTrue(crates.ssl_verify)

    def test_has_archiving(self):
        """Test if it returns True when has_archiving is called"""

        self.assertEqual(Crates.has_archiving(), True)

    def test_has_resuming(self):
        """Test if it returns True when has_resuming is called"""

        self.assertEqual(Crates.has_resuming(), False)

    @httpretty.activate
    def test_fetch_crates(self):
        """Test whether a list of crates is returned"""

        setup_http_server()

        backend = Crates()
        items = [items for items in backend.fetch(from_date=None)]

        self.assertEqual(len(items), 4)

        item = items[0]
        self.assertEqual(item['category'], CATEGORY_CRATES)
        self.assertEqual(len(item['data']['owner_team_data']['teams']), 0)
        self.assertEqual(len(item['data']['owner_user_data']['users']), 1)
        self.assertEqual(len(item['data']['version_downloads_data']['version_downloads']), 2)
        self.assertEqual(len(item['data']['versions_data']['versions']), 1)

        item = items[1]
        self.assertEqual(item['category'], CATEGORY_CRATES)
        self.assertEqual(len(item['data']['owner_team_data']['teams']), 1)
        self.assertEqual(len(item['data']['owner_user_data']['users']), 2)
        self.assertEqual(len(item['data']['version_downloads_data']), 0)
        self.assertEqual(len(item['data']['versions_data']['versions']), 5)

        item = items[2]
        self.assertEqual(item['category'], CATEGORY_CRATES)
        self.assertEqual(len(item['data']['owner_team_data']['teams']), 1)
        self.assertEqual(len(item['data']['owner_user_data']['users']), 2)
        self.assertEqual(len(item['data']['version_downloads_data']), 0)
        self.assertEqual(len(item['data']['versions_data']['versions']), 1)

        item = items[3]
        self.assertEqual(item['category'], CATEGORY_CRATES)
        self.assertEqual(len(item['data']['owner_team_data']['teams']), 1)
        self.assertEqual(len(item['data']['owner_user_data']['users']), 3)
        self.assertEqual(len(item['data']['version_downloads_data']), 0)
        self.assertEqual(len(item['data']['versions_data']['versions']), 7)

    @httpretty.activate
    def test_crates_search_fields(self):
        """Test whether the search_fields is properly set"""

        setup_http_server()

        backend = Crates()
        items = [items for items in backend.fetch(from_date=None)]

        for item in items:
            self.assertEqual(backend.metadata_id(item['data']), item['search_fields']['item_id'])

    @httpretty.activate
    def test_fetch_summary(self):
        """Test whether a summary is returned"""

        setup_http_server()

        backend = Crates()
        items = [items for items in backend.fetch(category=CATEGORY_SUMMARY)]

        self.assertEqual(len(items), 1)
        self.assertEqual(items[0]['category'], CATEGORY_SUMMARY)
        self.assertEqual(items[0]['data']['num_crates'], 10000)
        self.assertEqual(items[0]['data']['num_downloads'], 2000000000)

    @httpretty.activate
    def test_summary_search_fields(self):
        """Test whether the search_fields is properly set"""

        setup_http_server()

        backend = Crates()
        items = [items for items in backend.fetch(category=CATEGORY_SUMMARY)]

        for item in items:
            self.assertEqual(backend.metadata_id(item['data']), item['search_fields']['item_id'])

    @httpretty.activate
    def test_fetch_from_date(self):
        """Test when return from date"""

        setup_http_server()

        backend = Crates()
        from_date = datetime.datetime(2016, 1, 1)
        items = [items for items in backend.fetch(from_date=from_date)]

        item = items[0]
        self.assertEqual(len(item['data']['owner_team_data']['teams']), 1)
        self.assertEqual(len(item['data']['owner_user_data']['users']), 2)
        self.assertEqual(len(item['data']['version_downloads_data']), 0)
        self.assertEqual(len(item['data']['versions_data']['versions']), 5)

        item = items[1]
        self.assertEqual(len(item['data']['owner_team_data']['teams']), 1)
        self.assertEqual(len(item['data']['owner_user_data']['users']), 2)
        self.assertEqual(len(item['data']['version_downloads_data']), 0)
        self.assertEqual(len(item['data']['versions_data']['versions']), 1)

        item = items[2]
        self.assertEqual(len(item['data']['owner_team_data']['teams']), 1)
        self.assertEqual(len(item['data']['owner_user_data']['users']), 3)
        self.assertEqual(len(item['data']['version_downloads_data']), 0)
        self.assertEqual(len(item['data']['versions_data']['versions']), 7)

    @httpretty.activate
    def test_fetch_summary_from_date(self):
        """Test when return from date"""

        setup_http_server()

        backend = Crates()
        from_date = datetime.datetime(2016, 1, 1)
        items = [items for items in backend.fetch(category=CATEGORY_SUMMARY,
                                                  from_date=from_date)]

        self.assertEqual(len(items), 1)
        self.assertEqual(items[0]['category'], CATEGORY_SUMMARY)
        self.assertEqual(items[0]['data']['num_crates'], 10000)
        self.assertEqual(items[0]['data']['num_downloads'], 2000000000)

    @httpretty.activate
    def test_fetch_empty(self):
        """Test when return empty"""

        setup_http_server(empty=True)

        backend = Crates()
        items = [items for items in backend.fetch()]

        self.assertEqual(len(items), 0)


class TestCratesBackendArchive(TestCaseBackendArchive):
    """Crates backend tests using an archive"""

    def setUp(self):
        super().setUp()
        self.backend_write_archive = Crates(archive=self.archive)
        self.backend_read_archive = Crates(archive=self.archive)

    @httpretty.activate
    def test_fetch_crates_from_archive(self):
        """Test whether a list of crates is returned from archive"""

        setup_http_server()
        self._test_fetch_from_archive()

    @httpretty.activate
    def test_fetch_summary_from_archive(self):
        """Test whether a summary is returned from archive"""

        setup_http_server()
        items = [items for items in self.backend_write_archive.fetch(category=CATEGORY_SUMMARY)]
        items_archived = [items for items in self.backend_read_archive.fetch_from_archive()]

        self.assertEqual(len(items), len(items_archived))

        item = items[0]
        item_archived = items_archived[0]

        del item['data']['fetched_on']
        del item_archived['data']['fetched_on']

        self.assertEqual(item['data'], item_archived['data'])

    @httpretty.activate
    def test_fetch_from_date_from_archive(self):
        """Test when return from date from archive"""

        setup_http_server()

        from_date = datetime.datetime(2016, 1, 1)
        self._test_fetch_from_archive(from_date=from_date)

    @httpretty.activate
    def test_fetch_summary_from_date_from_archive(self):
        """Test when return from date from archive"""

        setup_http_server()

        from_date = datetime.datetime(2016, 1, 1)
        items = [items for items in self.backend_write_archive.fetch(category=CATEGORY_SUMMARY, from_date=from_date)]
        items_archived = [items for items in self.backend_read_archive.fetch_from_archive()]

        self.assertEqual(len(items), len(items_archived))

        item = items[0]
        item_archived = items_archived[0]

        del item['data']['fetched_on']
        del item_archived['data']['fetched_on']

        self.assertEqual(item['data'], item_archived['data'])

    @httpretty.activate
    def test_fetch_empty_from_archive(self):
        """Test when fetch from archive return nothing when the archive is empty"""

        setup_http_server(empty=True)
        self._test_fetch_from_archive()


class TestCratesClient(unittest.TestCase):
    """Crates API client tests"""

    def test_init(self):
        """Test initialization"""

        crates = CratesClient()

        self.assertEqual(crates.base_url, CRATES_API_URL)
        self.assertEqual(crates.max_retries, 5)
        self.assertEqual(crates.sleep_time, 60)
        self.assertIsNone(crates.archive)
        self.assertFalse(crates.from_archive)
        self.assertTrue(crates.ssl_verify)

        crates = CratesClient(ssl_verify=False)
        self.assertEqual(crates.base_url, CRATES_API_URL)
        self.assertEqual(crates.max_retries, 5)
        self.assertEqual(crates.sleep_time, 60)
        self.assertIsNone(crates.archive)
        self.assertFalse(crates.from_archive)
        self.assertFalse(crates.ssl_verify)

    @httpretty.activate
    def test_summary(self):
        """Test summary API call"""

        setup_http_server()

        client = CratesClient()
        summary = json.loads(client.summary())

        self.assertEqual(summary['num_crates'], 10000)
        self.assertEqual(summary['num_downloads'], 2000000000)

    @httpretty.activate
    def test_crates(self):
        """Test crates API call"""

        setup_http_server()

        client = CratesClient()
        crates = [crates for crates in client.crates()]
        self.assertEqual(len(crates), 2)

        # Check requests
        expected = {
            'sort': ['alphabetical'],
            'page': ['2']
        }

        self.assertDictEqual(httpretty.last_request().querystring, expected)

    @httpretty.activate
    def test_crate(self):
        """ Test crate API call """

        setup_http_server()

        client = CratesClient()
        crate = client.crate('a')

        self.assertNotEqual(crate, None)


class TestCratesCommand(unittest.TestCase):
    """CratesCommand unit tests"""

    def test_backend_class(self):
        """Test if the backend class is Launchpad"""

        self.assertIs(CratesCommand.BACKEND, Crates)

    def test_setup_cmd_parser(self):
        """Test if it parser object is correctly initialized"""

        parser = CratesCommand.setup_cmd_parser()
        self.assertIsInstance(parser, BackendCommandArgumentParser)
        self.assertEqual(parser._backend, Crates)

        args = ['--tag', 'test',
                '--from-date', '1970-01-01',
                '--category', 'summary',
                '--sleep-time', '600']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.tag, 'test')
        self.assertEqual(parsed_args.from_date, DEFAULT_DATETIME)
        self.assertEqual(parsed_args.category, CATEGORY_SUMMARY)
        self.assertEqual(parsed_args.sleep_time, 600)
        self.assertTrue(parsed_args.ssl_verify)

        args = ['--tag', 'test',
                '--from-date', '1970-01-01',
                '--category', 'summary',
                '--no-ssl-verify']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.tag, 'test')
        self.assertEqual(parsed_args.from_date, DEFAULT_DATETIME)
        self.assertEqual(parsed_args.category, CATEGORY_SUMMARY)
        self.assertEqual(parsed_args.sleep_time, SLEEP_TIME)
        self.assertFalse(parsed_args.ssl_verify)


if __name__ == "__main__":
    unittest.main(warnings='ignore')



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import json
import logging

import requests

from grimoirelab_toolkit.datetime import str_to_datetime
from grimoirelab_toolkit.uris import urijoin

from ...backend import (Backend,
                        BackendCommand,
                        BackendCommandArgumentParser)
from ...client import HttpClient
from ...errors import ParseError


logger = logging.getLogger(__name__)

KITSUNE_URL = "https://support.mozilla.org"
DEFAULT_OFFSET = 0

CATEGORY_QUESTION = "question"


class Kitsune(Backend):
    """Kitsune backend for Perceval.

    This class retrieves the questions and answers from a
    Kitsune URL. To initialize this class a URL may be provided.
    If not, https://support.mozilla.org will be used. The origin
    of the data will be set to this URL.

    Questions and answers are returned from older to newer.

    :param url: Kitsune URL
    :param tag: label used to mark the data
    :param archive: archive to store/retrieve items
    :param ssl_verify: enable/disable SSL verification
    """
    version = '0.8.0'

    CATEGORIES = [CATEGORY_QUESTION]

    def __init__(self, url=None, tag=None, archive=None, ssl_verify=True):
        if not url:
            url = KITSUNE_URL
        origin = url

        super().__init__(origin, tag=tag, archive=archive, ssl_verify=ssl_verify)
        self.url = url

        self.client = None

    def fetch(self, category=CATEGORY_QUESTION, offset=DEFAULT_OFFSET):
        """Fetch questions from the Kitsune url.

        :param category: the category of items to fetch
        :offset: obtain questions after offset
        :returns: a generator of questions
        """
        if not offset:
            offset = DEFAULT_OFFSET

        kwargs = {"offset": offset}
        items = super().fetch(category, **kwargs)

        return items

    def fetch_items(self, category, **kwargs):
        """Fetch questions from the Kitsune url

        :param category: the category of items to fetch
        :param kwargs: backend arguments

        :returns: a generator of items
        """
        offset = kwargs['offset']

        logger.info("Looking for questions at url '%s' using offset %s",
                    self.url, str(offset))

        nquestions = 0  # number of questions processed
        tquestions = 0  # number of questions from API data
        equestions = 0  # number of questions dropped by errors

        # Always get complete pages so the first item is always
        # the first one in the page
        page = int(offset / KitsuneClient.ITEMS_PER_PAGE)
        page_offset = page * KitsuneClient.ITEMS_PER_PAGE
        # drop questions from page before the offset
        drop_questions = offset - page_offset
        current_offset = offset

        questions_page = self.client.get_questions(offset)

        while True:
            try:
                raw_questions = next(questions_page)
            except StopIteration:
                break
            except requests.exceptions.HTTPError as e:
                # Continue with the next page if it is a 500 error
                if e.response.status_code == 500:
                    logger.exception(e)
                    logger.error("Problem getting Kitsune questions. "
                                 "Loosing %i questions. Going to the next page.",
                                 KitsuneClient.ITEMS_PER_PAGE)
                    equestions += KitsuneClient.ITEMS_PER_PAGE
                    current_offset += KitsuneClient.ITEMS_PER_PAGE
                    questions_page = self.client.get_questions(current_offset)
                    continue
                else:
                    # If it is another error just propagate the exception
                    raise e

            try:
                questions_data = json.loads(raw_questions)
                tquestions = questions_data['count']
                questions = questions_data['results']
            except (ValueError, KeyError) as ex:
                logger.error(ex)
                cause = ("Bad JSON format for mozilla_questions: %s" % (raw_questions))
                raise ParseError(cause=cause)

            for question in questions:
                if drop_questions > 0:
                    # Remove extra questions due to page base retrieval
                    drop_questions -= 1
                    continue
                question['offset'] = current_offset
                current_offset += 1
                question['answers_data'] = []
                for raw_answers in self.client.get_question_answers(question['id']):
                    answers = json.loads(raw_answers)['results']
                    question['answers_data'] += answers
                yield question
                nquestions += 1

            logger.debug("Questions: %i/%i", nquestions + offset, tquestions)

        logger.info("Total number of questions: %i (%i total)", nquestions, tquestions)
        logger.info("Questions with errors dropped: %i", equestions)

    def metadata(self, item, filter_classified=False):
        """Kitsune metadata.

        This method takes items overrides `metadata` method to add extra
        information related to Kitsune (offset of the question).

        :param item: an item fetched by a backend
        :param filter_classified: sets if classified fields were filtered
        """
        item = super().metadata(item, filter_classified=filter_classified)
        item['offset'] = item['data'].pop('offset')

        return item

    @classmethod
    def has_archiving(cls):
        """Returns whether it supports archiving items on the fetch process.

        :returns: this backend supports items archive
        """
        return True

    @classmethod
    def has_resuming(cls):
        """Returns whether it supports to resume the fetch process.

        :returns: this backend supports items resuming
        """
        return True

    @staticmethod
    def metadata_id(item):
        """Extracts the identifier from a Kitsune item."""

        return str(item['id'])

    @staticmethod
    def metadata_updated_on(item):
        """Extracts the update time from a Kitsune item.

        The timestamp is extracted from 'updated' field.
        This date is a UNIX timestamp but needs to be converted to
        a float value.

        :param item: item generated by the backend

        :returns: a UNIX timestamp
        """
        return float(str_to_datetime(item['updated']).timestamp())

    @staticmethod
    def metadata_category(item):
        """Extracts the category from a Kitsune item.

        This backend only generates one type of item which is
        'question'.
        """
        return CATEGORY_QUESTION

    def _init_client(self, from_archive=False):
        """Init client"""

        return KitsuneClient(self.url, self.archive, from_archive, self.ssl_verify)


class KitsuneClient(HttpClient):
    """Kitsune API client.

    This class implements a simple client to retrieve questions and answers from
    a Kitsune site.

    :param url: URL of Kitsune (sample https://support.mozilla.org)
    :param archive: an archive to store/read fetched data
    :param from_archive: it tells whether to write/read the archive
    :param ssl_verify: enable/disable SSL verification

    :raises HTTPError: when an error occurs doing the request
    """
    FIRST_PAGE = 1  # Initial page in Kitsune
    ITEMS_PER_PAGE = 20  # Items per page in Kitsune API

    def __init__(self, url, archive=None, from_archive=False, ssl_verify=True):
        super().__init__(urijoin(url, '/api/2/'), archive=archive, from_archive=from_archive, ssl_verify=ssl_verify)

    def get_questions(self, offset=None):
        """Retrieve questions from older to newer updated starting offset"""

        page = KitsuneClient.FIRST_PAGE

        if offset:
            page += int(offset / KitsuneClient.ITEMS_PER_PAGE)

        while True:
            api_questions_url = urijoin(self.base_url, '/question') + '/'

            params = {
                "page": page,
                "ordering": "updated"
            }

            questions = self.fetch(api_questions_url, params)
            yield questions

            questions_json = json.loads(questions)
            next_uri = questions_json['next']
            if not next_uri:
                break
            page += 1

    def get_question_answers(self, question_id):
        """Retrieve all answers for a question from older to newer (updated)"""

        page = KitsuneClient.FIRST_PAGE

        while True:
            api_answers_url = urijoin(self.base_url, '/answer') + '/'
            params = {
                "page": page,
                "question": question_id,
                "ordering": "updated"
            }

            answers_raw = self.fetch(api_answers_url, params)
            yield answers_raw

            answers = json.loads(answers_raw)
            if not answers['next']:
                break
            page += 1

    def fetch(self, url, params):
        """Return the textual content associated to the Response object"""

        logger.debug("Kitsune client calls API: %s params: %s",
                     url, str(params))

        response = super().fetch(url, payload=params)

        return response.text


class KitsuneCommand(BackendCommand):
    """Class to run Kitsune backend from the command line."""

    BACKEND = Kitsune

    @classmethod
    def setup_cmd_parser(cls):
        """Returns the Kitsune argument parser."""

        parser = BackendCommandArgumentParser(cls.BACKEND,
                                              offset=True,
                                              archive=True,
                                              ssl_verify=True)

        # Required arguments
        parser.parser.add_argument('url', nargs='?',
                                   default="https://support.mozilla.org",
                                   help="Kitsune URL (default: https://support.mozilla.org)")

        return parser



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Santiago Dueñas <sduenas@bitergia.com>
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import json
import unittest

import httpretty
import requests

from perceval.backend import BackendCommandArgumentParser

from perceval.backends.mozilla.kitsune import (Kitsune,
                                               KitsuneCommand,
                                               KitsuneClient)
from base import TestCaseBackendArchive


KITSUNE_SERVER_URL = 'http://example.com'
KITSUNE_API = KITSUNE_SERVER_URL + '/api/2/'
KITSUNE_API_QUESTION = KITSUNE_SERVER_URL + '/api/2/question/'
KITSUNE_API_ANSWER = KITSUNE_SERVER_URL + '/api/2/answer/'

KITSUNE_SERVER_FAIL_PAGE = 69
KITSUNE_ITEMS_PER_PAGE = 20


def read_file(filename, mode='r'):
    with open(filename, mode) as f:
        content = f.read()
    return content


class HTTPServer():

    requests_http = []  # requests done to the server

    @classmethod
    def routes(cls, empty=False):
        """Configure in http the routes to be served"""
        mozilla_questions_1 = read_file('data/kitsune/kitsune_questions_1_2.json')
        mozilla_questions_2 = read_file('data/kitsune/kitsune_questions_2_2.json')
        mozilla_questions_empty = read_file('data/kitsune/kitsune_questions_empty.json')
        mozilla_question_answers_1 = read_file('data/kitsune/kitsune_question_answers.json')
        mozilla_question_answers_empty = read_file('data/kitsune/kitsune_question_answers_empy.json')

        if empty:
            mozilla_questions_1 = mozilla_questions_empty

        def request_callback(method, uri, headers):
            page = uri.split("page=")[1].split("&")[0]
            if page == "1":
                if 'question/' in uri:
                    body = mozilla_questions_1
                else:
                    question = uri.split("question=")[1].split("&")[0]
                    body = mozilla_question_answers_1
                    body_json = json.loads(body)['results']
                    if body_json[0]['question'] != int(question):
                        # The answers are not for this question
                        body = mozilla_question_answers_empty
            elif page == "2":
                if 'question/' in uri:
                    body = mozilla_questions_2
                else:
                    body = mozilla_question_answers_1
            elif page == str(KITSUNE_SERVER_FAIL_PAGE):
                # To tests for Internal Server Error
                return (500, headers, '')
            elif page == str(KITSUNE_SERVER_FAIL_PAGE + 1):
                # Next page to the server fail returns questions
                return (200, headers, mozilla_questions_2)
            else:
                return (404, headers, '')

            HTTPServer.requests_http.append(httpretty.last_request())

            return (200, headers, body)

        httpretty.register_uri(httpretty.GET,
                               KITSUNE_API_QUESTION,
                               responses=[
                                   httpretty.Response(body=request_callback)
                               ])
        httpretty.register_uri(httpretty.GET,
                               KITSUNE_API_ANSWER,
                               responses=[
                                   httpretty.Response(body=request_callback)
                               ])


class TestKitsuneBackend(unittest.TestCase):
    """Kitsune backend tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        kitsune = Kitsune(KITSUNE_SERVER_URL, tag='test')

        self.assertEqual(kitsune.url, KITSUNE_SERVER_URL)
        self.assertEqual(kitsune.origin, KITSUNE_SERVER_URL)
        self.assertEqual(kitsune.tag, 'test')
        self.assertIsNone(kitsune.client)
        self.assertTrue(kitsune.ssl_verify)

        # When tag is empty or None it will be set to
        # the value in url
        kitsune = Kitsune(KITSUNE_SERVER_URL, ssl_verify=False)
        self.assertEqual(kitsune.url, KITSUNE_SERVER_URL)
        self.assertEqual(kitsune.origin, KITSUNE_SERVER_URL)
        self.assertEqual(kitsune.tag, KITSUNE_SERVER_URL)
        self.assertFalse(kitsune.ssl_verify)

        kitsune = Kitsune(KITSUNE_SERVER_URL, tag='')
        self.assertEqual(kitsune.url, KITSUNE_SERVER_URL)
        self.assertEqual(kitsune.origin, KITSUNE_SERVER_URL)
        self.assertEqual(kitsune.tag, KITSUNE_SERVER_URL)
        self.assertTrue(kitsune.ssl_verify)

    def test_has_archiving(self):
        """Test if it returns True when has_archiving is called"""

        self.assertEqual(Kitsune.has_archiving(), True)

    def test_has_resuming(self):
        """Test if it returns True when has_resuming is called"""

        self.assertEqual(Kitsune.has_resuming(), True)

    def __check_questions_contents(self, questions):
        self.assertEqual(questions[0]['data']['num_votes'], 2)
        self.assertEqual(questions[0]['data']['num_answers'], 0)
        self.assertEqual(questions[0]['data']['locale'], 'en-US')
        self.assertEqual(questions[0]['origin'], KITSUNE_SERVER_URL)
        self.assertEqual(questions[0]['uuid'], '8fa01e2aadf37c6f2fa300ce529fd1a23feac333')
        self.assertEqual(questions[0]['updated_on'], 1467798846.0)
        self.assertEqual(questions[0]['offset'], 0)
        self.assertEqual(questions[0]['category'], 'question')
        self.assertEqual(questions[0]['tag'], KITSUNE_SERVER_URL)

        if len(questions) > 1:
            self.assertEqual(questions[1]['data']['num_votes'], 1)
            self.assertEqual(questions[1]['data']['num_answers'], 0)
            self.assertEqual(questions[1]['data']['locale'], 'es')
            self.assertEqual(questions[1]['origin'], KITSUNE_SERVER_URL)
            self.assertEqual(questions[1]['uuid'], '7a3fbfb33cfaaa32b2f121994faf019a054a9a06')
            self.assertEqual(questions[1]['updated_on'], 1467798439.0)
            self.assertEqual(questions[1]['offset'], 1)
            self.assertEqual(questions[1]['category'], 'question')
            self.assertEqual(questions[1]['tag'], KITSUNE_SERVER_URL)

    @httpretty.activate
    def test_fetch(self):
        """Test whether the questions are returned"""

        HTTPServer.routes()

        # Test fetch questions with their reviews
        kitsune = Kitsune(KITSUNE_SERVER_URL)

        questions = [question for question in kitsune.fetch(offset=None)]

        self.assertEqual(len(questions), 4)

        self.__check_questions_contents(questions)

    @httpretty.activate
    def test_fetch_offset(self):
        """Test whether the questions are returned offset"""

        HTTPServer.routes()

        # Test fetch questions with their reviews
        kitsune = Kitsune(KITSUNE_SERVER_URL)

        # Get all questions
        offset = 0
        questions = [question for question in kitsune.fetch(offset=offset)]
        self.assertEqual(len(questions), 4)
        self.__check_questions_contents(questions)

        # 4 total questions minus 2, 2 questions returned
        offset = 2
        questions = [question for question in kitsune.fetch(offset=offset)]
        self.assertEqual(len(questions), 2)
        self.assertEqual(questions[0]['offset'], 2)
        self.assertEqual(questions[1]['offset'], 3)

        offset = 4
        questions = [question for question in kitsune.fetch(offset=offset)]
        self.assertEqual(len(questions), 0)

        # Get no questions: we have two pages
        offset = KitsuneClient.ITEMS_PER_PAGE * 2
        with self.assertRaises(requests.exceptions.HTTPError):
            questions = [question for question in kitsune.fetch(offset=offset)]
            self.assertEqual(len(questions), 0)

    @httpretty.activate
    def test_fetch_empty(self):
        """Test whether it works when no jobs are fetched"""

        HTTPServer.routes(empty=True)

        kitsune = Kitsune(KITSUNE_SERVER_URL)
        questions = [event for event in kitsune.fetch()]

        self.assertEqual(len(questions), 0)

    @httpretty.activate
    def test_fetch_server_error(self):
        """Test whether it works when the server fails"""

        HTTPServer.routes(empty=True)

        kitsune = Kitsune(KITSUNE_SERVER_URL)
        offset = (KITSUNE_SERVER_FAIL_PAGE - 1) * KITSUNE_ITEMS_PER_PAGE
        questions = [event for event in kitsune.fetch(offset=offset)]
        # After the failing page there are a page with 2 questions
        self.assertEqual(len(questions), 2)

    @httpretty.activate
    def test_search_fields(self):
        """Test whether the search_fields is properly set"""

        HTTPServer.routes()

        # Test fetch questions with their reviews
        kitsune = Kitsune(KITSUNE_SERVER_URL)

        questions = [question for question in kitsune.fetch(offset=None)]

        for question in questions:
            self.assertEqual(kitsune.metadata_id(question['data']), question['search_fields']['item_id'])


class TestKitsuneBackendArchive(TestCaseBackendArchive):
    """Kitsune backend tests using an archive"""

    def setUp(self):
        super().setUp()
        self.backend_write_archive = Kitsune(KITSUNE_SERVER_URL, archive=self.archive)
        self.backend_read_archive = Kitsune(KITSUNE_SERVER_URL, archive=self.archive)

    @httpretty.activate
    def test_fetch_from_archive(self):
        """Test whether the questions are returned from archive"""

        HTTPServer.routes()

        self._test_fetch_from_archive()

    @httpretty.activate
    def test_fetch_offset_from_archive_offset_0(self):
        """Test whether the questions are returned offset from archive"""

        HTTPServer.routes()

        offset = 0
        self._test_fetch_from_archive(offset=offset)

    @httpretty.activate
    def test_fetch_offset_from_archive_offset_2(self):
        """Test whether the questions are returned offset from archive"""

        HTTPServer.routes()

        offset = 2
        self._test_fetch_from_archive(offset=offset)

    @httpretty.activate
    def test_fetch_offset_from_archive_offset_4(self):
        """Test whether the questions are returned offset from archive"""

        HTTPServer.routes()

        offset = 4
        self._test_fetch_from_archive(offset=offset)

    @httpretty.activate
    def test_fetch_empty_from_archive(self):
        """Test whether it works when no jobs are fetched from archive"""

        HTTPServer.routes(empty=True)
        self._test_fetch_from_archive()


class TestKitsuneCommand(unittest.TestCase):
    """Tests for KitsuneCommand class"""

    def test_backend_class(self):
        """Test if the backend class is Kitsune"""

        self.assertIs(KitsuneCommand.BACKEND, Kitsune)

    def test_setup_cmd_parser(self):
        """Test if it parser object is correctly initialized"""

        parser = KitsuneCommand.setup_cmd_parser()
        self.assertIsInstance(parser, BackendCommandArgumentParser)
        self.assertEqual(parser._backend, Kitsune)

        args = [KITSUNE_SERVER_URL,
                '--tag', 'test',
                '--no-archive',
                '--offset', '88']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.url, KITSUNE_SERVER_URL)
        self.assertEqual(parsed_args.tag, 'test')
        self.assertEqual(parsed_args.no_archive, True)
        self.assertEqual(parsed_args.offset, 88)
        self.assertTrue(parsed_args.ssl_verify)

        args = [KITSUNE_SERVER_URL,
                '--no-ssl-verify']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.url, KITSUNE_SERVER_URL)
        self.assertFalse(parsed_args.ssl_verify)


class TestKitsuneClient(unittest.TestCase):
    """Kitsune API client tests

    These tests not check the body of the response, only if the call
    was well formed and if a response was obtained. Due to this, take
    into account that the body returned on each request might not
    match with the parameters from the request.
    """

    def test_init(self):
        """Test initialization"""

        base_url = KITSUNE_SERVER_URL + "/api/2"

        kitsune = KitsuneClient(KITSUNE_SERVER_URL)

        self.assertEqual(kitsune.base_url, base_url)
        self.assertIsNone(kitsune.archive)
        self.assertFalse(kitsune.from_archive)
        self.assertTrue(kitsune.ssl_verify)

        kitsune = KitsuneClient(KITSUNE_SERVER_URL, ssl_verify=False)

        self.assertEqual(kitsune.base_url, base_url)
        self.assertIsNone(kitsune.archive)
        self.assertFalse(kitsune.from_archive)
        self.assertFalse(kitsune.ssl_verify)

    @httpretty.activate
    def test_get_questions(self):
        """Test get_questions API call"""

        HTTPServer.routes()

        # Set up a mock HTTP server
        body = read_file('data/kitsune/kitsune_questions_1_2.json')
        client = KitsuneClient(KITSUNE_SERVER_URL)
        response = next(client.get_questions())  # first group of questions
        req = HTTPServer.requests_http[-1]
        self.assertEqual(response, body)
        self.assertEqual(req.method, 'GET')
        self.assertRegex(req.path, '/api/2/question/')
        # Check request params
        expected = {
            'page': ['1'],
            'ordering': ['updated'],
        }
        self.assertDictEqual(req.querystring, expected)

    @httpretty.activate
    def test_get_question_answers(self):
        """Test get_question_answers API call"""

        HTTPServer.routes()

        # Set up a mock HTTP server
        body = read_file('data/kitsune/kitsune_question_answers.json')
        client = KitsuneClient(KITSUNE_SERVER_URL)
        question_id = 1129949
        response = next(client.get_question_answers(question_id))
        req = HTTPServer.requests_http[-1]
        self.assertEqual(response, body)
        self.assertEqual(req.method, 'GET')
        self.assertRegex(req.path, '/api/2/answer/')
        # Check request params
        expected = {
            'question': ['1129949'],
            'page': ['1'],
            'ordering': ['updated']
        }
        self.assertDictEqual(req.querystring, expected)


if __name__ == "__main__":
    unittest.main(warnings='ignore')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Santiago Dueñas <sduenas@bitergia.com>
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import re
import unittest

import httpretty

from perceval.backend import BackendError, BackendCommandArgumentParser
from perceval.backends.mozilla.remo import (ReMo,
                                            ReMoCommand,
                                            ReMoClient,
                                            MOZILLA_REPS_URL)
from base import TestCaseBackendArchive

MOZILLA_REPS_SERVER_URL = 'http://example.com'
MOZILLA_REPS_API = MOZILLA_REPS_SERVER_URL + '/api/remo/v1'

MOZILLA_REPS_CATEGORIES = ['events', 'activities', 'users']


def read_file(filename, mode='r'):
    with open(filename, mode) as f:
        content = f.read()
    return content


class HTTPServer():

    requests_http = []  # requests done to the server

    @classmethod
    def routes(cls, empty=False):
        """Configure in http the routes to be served"""

        mozilla_bodies = {}  # dict with all the bodies to be returned by category
        for category in MOZILLA_REPS_CATEGORIES:
            mozilla_bodies[category] = {}
            # First two pages for each category to test pagination
            mozilla_bodies[category]['1'] = read_file('data/remo/remo_' + category + '_page_1_2.json')
            mozilla_bodies[category]['2'] = read_file('data/remo/remo_' + category + '_page_2_2.json')
            # A sample item per each category
            mozilla_bodies[category]['item'] = read_file('data/remo/remo_' + category + '.json')

        if empty:
            for category in MOZILLA_REPS_CATEGORIES:
                mozilla_bodies[category]['1'] = read_file('data/remo/remo_' + category + '_page_empty.json')

        def request_callback(method, uri, headers):
            body = ''
            if 'page' in uri:
                # Page with item list query
                page = uri.split("page=")[1].split("&")[0]
                for category in MOZILLA_REPS_CATEGORIES:
                    if category in uri:
                        body = mozilla_bodies[category][page]
                        break
            else:
                # Specific item. Always return the same for each category.
                for category in MOZILLA_REPS_CATEGORIES:
                    if category in uri:
                        body = mozilla_bodies[category]['item']
                        break

            HTTPServer.requests_http.append(httpretty.last_request())

            return (200, headers, body)

        httpretty.register_uri(httpretty.GET,
                               re.compile(MOZILLA_REPS_API + ".*"),
                               responses=[
                                   httpretty.Response(body=request_callback)
                               ])


class TestReMoBackend(unittest.TestCase):
    """ReMo backend tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        remo = ReMo(MOZILLA_REPS_SERVER_URL, tag='test')

        self.assertEqual(remo.url, MOZILLA_REPS_SERVER_URL)
        self.assertEqual(remo.origin, MOZILLA_REPS_SERVER_URL)
        self.assertEqual(remo.tag, 'test')
        self.assertIsNone(remo.client)
        self.assertTrue(remo.ssl_verify)

        # When tag is empty or None it will be set to
        # the value in url
        remo = ReMo(MOZILLA_REPS_SERVER_URL, ssl_verify=False)
        self.assertEqual(remo.url, MOZILLA_REPS_SERVER_URL)
        self.assertEqual(remo.origin, MOZILLA_REPS_SERVER_URL)
        self.assertEqual(remo.tag, MOZILLA_REPS_SERVER_URL)
        self.assertFalse(remo.ssl_verify)

        remo = ReMo(MOZILLA_REPS_SERVER_URL, tag='')
        self.assertEqual(remo.url, MOZILLA_REPS_SERVER_URL)
        self.assertEqual(remo.origin, MOZILLA_REPS_SERVER_URL)
        self.assertEqual(remo.tag, MOZILLA_REPS_SERVER_URL)
        self.assertTrue(remo.ssl_verify)

        # If no url is provided, MOZILLA_REPS_URL is used
        remo = ReMo()
        self.assertEqual(remo.url, MOZILLA_REPS_URL)
        self.assertEqual(remo.origin, MOZILLA_REPS_URL)
        self.assertEqual(remo.tag, MOZILLA_REPS_URL)

    def test_has_archiving(self):
        """Test if it returns True when has_archiving is called"""

        self.assertEqual(ReMo.has_archiving(), True)

    def test_has_resuming(self):
        """Test if it returns True when has_resuming is called"""

        self.assertEqual(ReMo.has_resuming(), True)

    def __check_events_contents(self, items):
        self.assertEqual(items[0]['data']['city'], 'Makassar')
        self.assertEqual(items[0]['data']['end'], '2012-06-10T11:00:00Z')
        self.assertEqual(items[0]['origin'], MOZILLA_REPS_SERVER_URL)
        self.assertEqual(items[0]['uuid'], 'e701d4ed3b954361383d678d2168a44307d7ff60')
        self.assertEqual(items[0]['updated_on'], 1339326000.0)
        self.assertEqual(items[0]['category'], 'event')
        self.assertEqual(items[0]['tag'], MOZILLA_REPS_SERVER_URL)
        self.assertEqual(items[0]['offset'], 0)
        self.assertEqual(items[3]['offset'], 3)

    def __check_activities_contents(self, items):
        self.assertEqual(items[0]['data']['location'], 'Bhopal, Madhya Pradesh, India')
        self.assertEqual(items[0]['data']['report_date'], '2016-11-05')
        self.assertEqual(items[0]['origin'], MOZILLA_REPS_SERVER_URL)
        self.assertEqual(items[0]['uuid'], '9e2b0c2c8ec8094d2c53a2621bd09f9d6f65e67f')
        self.assertEqual(items[0]['updated_on'], 1478304000.0)
        self.assertEqual(items[0]['category'], 'activity')
        self.assertEqual(items[0]['tag'], MOZILLA_REPS_SERVER_URL)
        self.assertEqual(items[0]['offset'], 0)
        self.assertEqual(items[3]['offset'], 3)

    def __check_users_contents(self, items):
        self.assertEqual(items[0]['data']['city'], 'Makati City')
        self.assertEqual(items[0]['data']['date_joined_program'], '2011-06-01')
        self.assertEqual(items[0]['origin'], MOZILLA_REPS_SERVER_URL)
        self.assertEqual(items[0]['uuid'], '90b0f5bc90ed8a694261df418a2b85beed94535a')
        self.assertEqual(items[0]['updated_on'], 1306886400.0)
        self.assertEqual(items[0]['category'], 'user')
        self.assertEqual(items[0]['tag'], MOZILLA_REPS_SERVER_URL)
        self.assertEqual(items[0]['offset'], 0)
        self.assertEqual(items[3]['offset'], 3)

    @httpretty.activate
    def __test_fetch(self, category='event'):
        """Test whether the events are returned"""

        items_page = ReMoClient.ITEMS_PER_PAGE
        pages = 2  # two pages of testing data

        HTTPServer.routes()
        prev_requests_http = len(HTTPServer.requests_http)

        # Test fetch events with their reviews
        remo = ReMo(MOZILLA_REPS_SERVER_URL)

        items = [page for page in remo.fetch(offset=None, category=category)]

        self.assertEqual(len(items), items_page * pages)

        if category == 'event':
            self.__check_events_contents(items)
        elif category == 'user':
            self.__check_users_contents(items)
        elif category == 'activity':
            self.__check_activities_contents(items)

        # Check requests: page list, items, page list, items
        expected = [{'page': ['1'], 'orderby': ['ASC']}]
        for i in range(0, items_page):
            expected += [{}]
        expected += [{'page': ['2'], 'orderby': ['ASC']}]
        for i in range(0, items_page):
            expected += [{}]

        self.assertEqual(len(HTTPServer.requests_http) - prev_requests_http, len(expected))

        for i in range(len(expected)):
            self.assertDictEqual(HTTPServer.requests_http[i].querystring, expected[i])

    @httpretty.activate
    def __test_search_terms(self, category='event'):
        """Test whether the search_fields is properly set"""

        items_page = ReMoClient.ITEMS_PER_PAGE
        pages = 2  # two pages of testing data

        HTTPServer.routes()

        # Test fetch events with their reviews
        remo = ReMo(MOZILLA_REPS_SERVER_URL)

        items = [page for page in remo.fetch(offset=None, category=category)]

        for item in items:
            self.assertEqual(remo.metadata_id(item['data']), item['search_fields']['item_id'])

    def test_fetch_events(self):
        self.__test_fetch(category='event')

    def test_fetch_activities(self):
        self.__test_fetch(category='activity')

    def test_fetch_users(self):
        self.__test_fetch(category='user')

    def test_search_terms_events(self):
        self.__test_search_terms(category='event')

    def test_search_terms_activities(self):
        self.__test_search_terms(category='activity')

    def test_search_terms_users(self):
        self.__test_search_terms(category='user')

    @httpretty.activate
    def tests_wrong_metadata_updated_on(self):

        HTTPServer.routes()
        prev_requests_http = len(HTTPServer.requests_http)

        # Test fetch events with their reviews
        remo = ReMo(MOZILLA_REPS_SERVER_URL)

        items = [page for page in remo.fetch(offset=None, category="event")]
        item = items[0]

        item.pop('end', None)

        with self.assertRaises(ValueError):
            remo.metadata_updated_on(item)

    @httpretty.activate
    def tests_wrong_metadata_category(self):

        HTTPServer.routes()
        prev_requests_http = len(HTTPServer.requests_http)

        # Test fetch events with their reviews
        remo = ReMo(MOZILLA_REPS_SERVER_URL)

        items = [page for page in remo.fetch(offset=None, category="event")]
        item = items[0]

        item.pop('estimated_attendance', None)

        with self.assertRaises(TypeError):
            remo.metadata_category(item)

    @httpretty.activate
    def test_fetch_offset(self):
        items_page = ReMoClient.ITEMS_PER_PAGE
        pages = 2  # two pages of testing data
        offset = 15

        HTTPServer.routes()
        prev_requests_http = len(HTTPServer.requests_http)

        # Test fetch events with their reviews
        remo = ReMo(MOZILLA_REPS_SERVER_URL)

        # Test we get the correct number of items from an offset
        items = [page for page in remo.fetch(offset=15)]
        self.assertEqual(len(items), (items_page * pages) - offset)

        # Test that the same offset (17) is the same item
        items = [page for page in remo.fetch(offset=5)]
        uuid_17_1 = items[12]['uuid']
        self.assertEqual(items[12]['offset'], 17)
        items = [page for page in remo.fetch(offset=12)]
        uuid_17_2 = items[5]['uuid']
        self.assertEqual(items[5]['offset'], 17)
        self.assertEqual(uuid_17_1, uuid_17_2)

    def test_fetch_wrong_category(self):
        with self.assertRaises(BackendError):
            self.__test_fetch(category='wrong')

    @httpretty.activate
    def test_fetch_empty(self):
        """Test whether it works when no items are fetched"""

        HTTPServer.routes(empty=True)

        remo = ReMo(MOZILLA_REPS_SERVER_URL)
        events = [event for event in remo.fetch()]

        self.assertEqual(len(events), 0)


class TestRemoBackendArchive(TestCaseBackendArchive):
    """Remo backend tests using an archive"""

    def setUp(self):
        super().setUp()
        self.backend_write_archive = ReMo(MOZILLA_REPS_SERVER_URL, archive=self.archive)
        self.backend_read_archive = ReMo(MOZILLA_REPS_SERVER_URL, archive=self.archive)

    @httpretty.activate
    def __test_fetch_from_archive(self, category='event'):
        """Test whether the events are returned from archive"""

        HTTPServer.routes()
        self._test_fetch_from_archive(category=category)

    def test_fetch_events(self):
        self.__test_fetch_from_archive(category='event')

    def test_fetch_activities(self):
        self.__test_fetch_from_archive(category='activity')

    def test_fetch_users(self):
        self.__test_fetch_from_archive(category='user')

    @httpretty.activate
    def test_fetch_offset_from_archive_15(self):
        """Test whether the events with offset are returned from archive"""

        HTTPServer.routes()
        self._test_fetch_from_archive(offset=15)

    @httpretty.activate
    def test_fetch_offset_from_archive_5(self):
        """Test whether the events with offset are returned from archive"""

        HTTPServer.routes()
        self._test_fetch_from_archive(offset=5)

    @httpretty.activate
    def test_fetch_empty_from_archive(self):
        """Test whether it works when no items are fetched from archive"""

        HTTPServer.routes(empty=True)
        self._test_fetch_from_archive()


class TestReMoCommand(unittest.TestCase):
    """Tests for ReMoCommand class"""

    def test_backend_class(self):
        """Test if the backend class is ReMo"""

        self.assertIs(ReMoCommand.BACKEND, ReMo)

    def test_setup_cmd_parser(self):
        """Test if it parser object is correctly initialized"""

        parser = ReMoCommand.setup_cmd_parser()
        self.assertIsInstance(parser, BackendCommandArgumentParser)
        self.assertEqual(parser._backend, ReMo)

        args = [MOZILLA_REPS_SERVER_URL,
                '--category', 'user',
                '--tag', 'test',
                '--no-archive',
                '--offset', '88']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.url, MOZILLA_REPS_SERVER_URL)
        self.assertEqual(parsed_args.category, 'user')
        self.assertEqual(parsed_args.tag, 'test')
        self.assertEqual(parsed_args.no_archive, True)
        self.assertEqual(parsed_args.offset, 88)
        self.assertTrue(parsed_args.ssl_verify)

        args = [MOZILLA_REPS_SERVER_URL, '--no-ssl-verify']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.url, MOZILLA_REPS_SERVER_URL)
        self.assertFalse(parsed_args.ssl_verify)


class TestReMoClient(unittest.TestCase):
    """ReMo API client tests

    These tests not check the body of the response, only if the call
    was well formed and if a response was obtained. Due to this, take
    into account that the body returned on each request might not
    match with the parameters from the request.
    """

    def test_init(self):
        """Test initialization"""

        remo = ReMoClient(MOZILLA_REPS_SERVER_URL)

        self.assertEqual(remo.base_url, MOZILLA_REPS_SERVER_URL)
        self.assertIsNone(remo.archive)
        self.assertFalse(remo.from_archive)
        self.assertTrue(remo.ssl_verify)

        remo = ReMoClient(MOZILLA_REPS_SERVER_URL, ssl_verify=False)

        self.assertEqual(remo.base_url, MOZILLA_REPS_SERVER_URL)
        self.assertIsNone(remo.archive)
        self.assertFalse(remo.from_archive)
        self.assertFalse(remo.ssl_verify)

    @httpretty.activate
    def test_get_items(self):
        """Test get_events API call"""

        HTTPServer.routes()

        # Set up a mock HTTP server
        body = read_file('data/remo/remo_events_page_1_2.json')
        client = ReMoClient(MOZILLA_REPS_SERVER_URL)
        response = next(client.get_items())
        req = HTTPServer.requests_http[-1]
        self.assertEqual(response, body)
        self.assertEqual(req.method, 'GET')
        self.assertRegex(req.path, '/api/remo/v1/events/')
        self.assertDictEqual(req.querystring, {'page': ['1'], 'orderby': ['ASC']})

    @httpretty.activate
    def test_get_wrong_items(self):
        """Test get_events API call"""

        HTTPServer.routes()

        # Set up a mock HTTP server
        body = read_file('data/remo/remo_events_page_1_2.json')
        client = ReMoClient(MOZILLA_REPS_SERVER_URL)

        with self.assertRaises(ValueError):
            _ = next(client.get_items(category='wrong'))

    @httpretty.activate
    def test_call(self):
        """Test get_all_users API call"""

        HTTPServer.routes()

        # Set up a mock HTTP server
        body = read_file('data/remo/remo_events_page_1_2.json')
        client = ReMoClient(MOZILLA_REPS_SERVER_URL)
        response = client.fetch(MOZILLA_REPS_API + '/events/?page=1')
        req = HTTPServer.requests_http[-1]
        self.assertEqual(response, body)
        self.assertEqual(req.method, 'GET')
        self.assertEqual(req.path, '/api/remo/v1/events/?page=1')
        # Check request params
        expected = {
            'page': ['1']
        }
        self.assertDictEqual(req.querystring, expected)


if __name__ == "__main__":
    unittest.main(warnings='ignore')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>
#     Santiago Dueñas <sduenas@bitergia.com>
#

import os
import shutil
import tempfile
import unittest

from perceval.archive import Archive


class TestCaseBackendArchive(unittest.TestCase):
    """Unit tests for Backend using the archive"""

    def setUp(self):
        self.test_path = tempfile.mkdtemp(prefix='perceval-mozilla_')
        archive_path = os.path.join(self.test_path, 'myarchive')
        self.archive = Archive.create(archive_path)

    def tearDown(self):
        shutil.rmtree(self.test_path)

    def _test_fetch_from_archive(self, **kwargs):
        """Test whether the method fetch_from_archive works properly"""

        items = [items for items in self.backend_write_archive.fetch(**kwargs)]
        items_archived = [item for item in self.backend_write_archive.fetch_from_archive()]

        self.assertEqual(len(items), len(items_archived))

        for i in range(len(items)):
            item = items[i]
            archived_item = items_archived[i]

            del item['timestamp']
            del archived_item['timestamp']

            self.assertEqual(item, archived_item)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import json
import logging

import requests

from grimoirelab_toolkit.datetime import (datetime_utcnow,
                                          datetime_to_utc,
                                          str_to_datetime)
from grimoirelab_toolkit.uris import urijoin

from ...backend import (Backend,
                        BackendCommand,
                        BackendCommandArgumentParser)
from ...client import HttpClient
from ...utils import DEFAULT_DATETIME

CRATES_URL = "https://crates.io/"
CRATES_API_URL = 'https://crates.io/api/v1/'

CATEGORY_CRATES = 'crates'
CATEGORY_SUMMARY = 'summary'

SLEEP_TIME = 60

logger = logging.getLogger(__name__)


class Crates(Backend):
    """Crates.io backend for Perceval.

    This class allows the fetch the packages stored in Crates.io

    :param sleep_time: sleep time in case of connection lost
    :param tag: label used to mark the data
    :param archive: archive to store/retrieve items
    :param ssl_verify: enable/disable SSL verification
    """
    version = '0.5.0'

    CATEGORIES = [CATEGORY_CRATES, CATEGORY_SUMMARY]

    def __init__(self, sleep_time=SLEEP_TIME, tag=None, archive=None, ssl_verify=True):
        origin = CRATES_URL

        super().__init__(origin, tag=tag, archive=archive, ssl_verify=ssl_verify)
        self.sleep_time = sleep_time

        self.client = None

    def fetch(self, category=CATEGORY_CRATES, from_date=DEFAULT_DATETIME):
        """Fetch package data.

        The method retrieves packages and summary from Crates.io.

        :param category: the category of items to fetch
        :param from_date: obtain packages updated since this date

        :returns: a summary and crate items
        """
        if not from_date:
            from_date = DEFAULT_DATETIME

        from_date = datetime_to_utc(from_date)

        kwargs = {"from_date": from_date}
        items = super().fetch(category, **kwargs)

        return items

    def fetch_items(self, category, **kwargs):
        """Fetch packages and summary from Crates.io

        :param category: the category of items to fetch
        :param kwargs: backend arguments

        :returns: a generator of items
        """
        from_date = kwargs['from_date']

        if category == CATEGORY_CRATES:
            return self.__fetch_crates(from_date)
        else:
            return self.__fetch_summary()

    @classmethod
    def has_archiving(cls):
        """Returns whether it supports archiving items on the fetch process.

        :returns: this backend supports items archive
        """
        return True

    @classmethod
    def has_resuming(cls):
        """Returns whether it supports to resume the fetch process.

        :returns: this backend supports items resuming
        """
        return False

    @staticmethod
    def metadata_id(item):
        """Extracts the identifier from an item depending on its type."""

        if Crates.metadata_category(item) == CATEGORY_CRATES:
            return str(item['id'])
        else:
            ts = item['fetched_on']
            ts = str_to_datetime(ts)
            return str(ts.timestamp())

    @staticmethod
    def metadata_updated_on(item):
        """Extracts the update time from an item.

        Depending on the item, the timestamp is extracted from the
        'updated_at' or 'fetched_on' fields.
        This date is converted to UNIX timestamp format.

        :param item: item generated by the backend

        :returns: a UNIX timestamp
        """
        if Crates.metadata_category(item) == CATEGORY_CRATES:
            ts = item['updated_at']
        else:
            ts = item['fetched_on']

        ts = str_to_datetime(ts)

        return ts.timestamp()

    @staticmethod
    def metadata_category(item):
        """Extracts the category from an item.

        This backend generates two types of item: 'summary' and 'crate'.
        """
        if 'num_downloads' in item:
            return CATEGORY_SUMMARY
        else:
            return CATEGORY_CRATES

    def _init_client(self, from_archive=False):
        """Init client"""

        return CratesClient(self.sleep_time, self.archive, from_archive, self.ssl_verify)

    def __fetch_summary(self):
        """Fetch summary"""

        raw_summary = self.client.summary()
        summary = json.loads(raw_summary)
        summary['fetched_on'] = str(datetime_utcnow())

        yield summary

    def __fetch_crates(self, from_date):
        """Fetch crates"""

        from_date = datetime_to_utc(from_date)

        crates_groups = self.client.crates()

        for raw_crates in crates_groups:
            crates = json.loads(raw_crates)

            for crate_container in crates['crates']:

                if str_to_datetime(crate_container['updated_at']) < from_date:
                    continue

                crate_id = crate_container['id']

                crate = self.__fetch_crate_data(crate_id)
                crate['owner_team_data'] = self.__fetch_crate_owner_team(crate_id)
                crate['owner_user_data'] = self.__fetch_crate_owner_user(crate_id)
                crate['version_downloads_data'] = self.__fetch_crate_version_downloads(crate_id)
                crate['versions_data'] = self.__fetch_crate_versions(crate_id)

                yield crate

    def __fetch_crate_owner_team(self, crate_id):
        """Get crate team owner"""

        raw_owner_team = self.client.crate_attribute(crate_id, 'owner_team')

        owner_team = json.loads(raw_owner_team)

        return owner_team

    def __fetch_crate_owner_user(self, crate_id):
        """Get crate user owners"""

        raw_owner_user = self.client.crate_attribute(crate_id, 'owner_user')

        owner_user = json.loads(raw_owner_user)

        return owner_user

    def __fetch_crate_versions(self, crate_id):
        """Get crate versions data"""

        raw_versions = self.client.crate_attribute(crate_id, "versions")

        version_downloads = json.loads(raw_versions)

        return version_downloads

    def __fetch_crate_version_downloads(self, crate_id):
        """Get crate version downloads"""

        raw_version_downloads = self.client.crate_attribute(crate_id, "downloads")

        version_downloads = json.loads(raw_version_downloads)

        return version_downloads

    def __fetch_crate_data(self, crate_id):
        """Get crate data"""

        raw_crate = self.client.crate(crate_id)

        crate = json.loads(raw_crate)
        return crate['crate']


class CratesClient(HttpClient):
    """Crates API client.

    Client for fetching information from the Crates API.

    :param sleep_time: time to sleep in case
        of connection problems
    :param archive: an archive to store/read fetched data
    :param from_archive: it tells whether to write/read the archive
    :param ssl_verify: enable/disable SSL verification
    """

    MAX_RETRIES = 5

    def __init__(self, sleep_time=SLEEP_TIME, archive=None, from_archive=False, ssl_verify=True):
        super().__init__(CRATES_API_URL, sleep_time=sleep_time, max_retries=CratesClient.MAX_RETRIES,
                         extra_headers={'Content-type': 'application/json'},
                         archive=archive, from_archive=from_archive, ssl_verify=ssl_verify)

    def summary(self):
        """Get Crates.io summary"""

        path = urijoin(CRATES_API_URL, CATEGORY_SUMMARY)
        raw_content = self.fetch(path)

        return raw_content

    def crates(self, from_page=1):
        """Get crates in alphabetical order"""

        path = urijoin(CRATES_API_URL, CATEGORY_CRATES)
        raw_crates = self.__fetch_items(path, from_page)

        return raw_crates

    def crate(self, crate_id):
        """Get a crate by its ID"""

        path = urijoin(CRATES_API_URL, CATEGORY_CRATES, crate_id)
        raw_crate = self.fetch(path)

        return raw_crate

    def crate_attribute(self, crate_id, attribute):
        """Get crate attribute"""

        path = urijoin(CRATES_API_URL, CATEGORY_CRATES, crate_id, attribute)
        raw_attribute_data = self.fetch(path)

        return raw_attribute_data

    def __fetch_items(self, path, page=1):
        """Return the items from Crates.io API using pagination"""

        fetch_data = True
        parsed_crates = 0
        total_crates = 0

        while fetch_data:
            logger.debug("Fetching page: %i", page)

            try:
                payload = {'sort': 'alphabetical', 'page': page}
                raw_content = self.fetch(path, payload=payload)
                content = json.loads(raw_content)

                parsed_crates += len(content['crates'])

                if not total_crates:
                    total_crates = content['meta']['total']

            except requests.exceptions.HTTPError as e:
                logger.error("HTTP exception raised - %s", e.response.text)
                raise e

            yield raw_content
            page += 1

            if parsed_crates >= total_crates:
                fetch_data = False

    def fetch(self, url, payload=None):
        """Return the textual content associated to the Response object"""

        response = super().fetch(url, payload=payload)

        return response.text


class CratesCommand(BackendCommand):
    """Class to run Crates.io backend from the command line."""

    BACKEND = Crates

    @classmethod
    def setup_cmd_parser(cls):
        """Returns the Crates argument parser."""

        parser = BackendCommandArgumentParser(cls.BACKEND,
                                              from_date=True,
                                              archive=True,
                                              token_auth=True,
                                              ssl_verify=True)

        # Optional arguments
        group = parser.parser.add_argument_group('Crates.io arguments')
        group.add_argument('--sleep-time', dest='sleep_time',
                           default=SLEEP_TIME, type=int,
                           help="Sleep time in case of connection lost")

        return parser



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import json
import logging
import urllib.parse

from grimoirelab_toolkit.datetime import str_to_datetime
from grimoirelab_toolkit.uris import urijoin

from ...backend import (Backend,
                        BackendCommand,
                        BackendCommandArgumentParser)
from ...client import HttpClient

MOZILLA_REPS_URL = "https://reps.mozilla.org"
REMO_DEFAULT_OFFSET = 0

CATEGORY_EVENT = 'event'
CATEGORY_ACTIVITY = 'activity'
CATEGORY_USER = 'user'

logger = logging.getLogger(__name__)


class ReMo(Backend):
    """ReMo backend for Perceval.

    This class retrieves the events from a ReMo URL. To initialize
    this class an URL may be provided. If not, https://reps.mozilla.org
    will be used. The origin of the data will be set to this URL.

    It uses v2 API to get events, people and activities data.

    :param url: ReMo URL
    :param tag: label used to mark the data
    :param archive: archive to store/retrieve items
    :param ssl_verify: enable/disable SSL verification
    """
    version = '0.9.0'

    CATEGORIES = [CATEGORY_ACTIVITY, CATEGORY_EVENT, CATEGORY_USER]

    def __init__(self, url=None, tag=None, archive=None, ssl_verify=True):
        if not url:
            url = MOZILLA_REPS_URL
        origin = url

        super().__init__(origin, tag=tag, archive=archive, ssl_verify=ssl_verify)
        self.url = url
        self.client = None

        self.__users = {}  # internal users cache

    def fetch(self, category=CATEGORY_EVENT, offset=REMO_DEFAULT_OFFSET):
        """Fetch items from the ReMo url.

        The method retrieves, from a ReMo URL, the set of items
        of the given `category`.

        :param category: the category of items to fetch
        :param offset: obtain items after offset
        :returns: a generator of items
        """
        if not offset:
            offset = REMO_DEFAULT_OFFSET

        kwargs = {"offset": offset}
        items = super().fetch(category, **kwargs)

        return items

    def fetch_items(self, category, **kwargs):
        """Fetch items

        :param category: the category of items to fetch
        :param kwargs: backend arguments

        :returns: a generator of items
        """
        offset = kwargs['offset']

        logger.info("Looking for events at url '%s' of %s category and %i offset",
                    self.url, category, offset)

        nitems = 0  # number of items processed
        titems = 0  # number of items from API data

        # Always get complete pages so the first item is always
        # the first one in the page
        page = int(offset / ReMoClient.ITEMS_PER_PAGE)
        page_offset = page * ReMoClient.ITEMS_PER_PAGE
        # drop items from page before the offset
        drop_items = offset - page_offset
        logger.debug("%i items dropped to get %i offset starting in page %i (%i page offset)",
                     drop_items, offset, page, page_offset)
        current_offset = offset

        for raw_items in self.client.get_items(category, offset):
            items_data = json.loads(raw_items)
            titems = items_data['count']
            logger.info("Pending items to retrieve: %i, %i current offset",
                        titems - current_offset, current_offset)
            items = items_data['results']
            for item in items:
                if drop_items > 0:
                    # Remove extra items due to page base retrieval
                    drop_items -= 1
                    continue
                raw_item_details = self.client.fetch(item['_url'])
                item_details = json.loads(raw_item_details)
                item_details['offset'] = current_offset
                current_offset += 1
                yield item_details
                nitems += 1

        logger.info("Total number of events: %i (%i total, %i offset)", nitems, titems, offset)

    def metadata(self, item, filter_classified=False):
        """ReMo metadata.

        This method takes items overrides `metadata` method to add extra
        information related to Remo (offset of the item).

        :param item: an item fetched by a backend
        :param filter_classified: sets if classified fields were filtered
        """
        item = super().metadata(item, filter_classified=filter_classified)
        item['offset'] = item['data'].pop('offset')

        return item

    @classmethod
    def has_archiving(cls):
        """Returns whether it supports archiving items on the fetch process.

        :returns: this backend supports items archive
        """
        return True

    @classmethod
    def has_resuming(cls):
        """Returns whether it supports to resume the fetch process.

        :returns: this backend supports items resuming
        """
        return True

    @staticmethod
    def metadata_id(item):
        """Extracts the identifier from a ReMo item."""
        return str(item['remo_url'])

    @staticmethod
    def metadata_updated_on(item):
        """Extracts the update time from a ReMo item.

        The timestamp is extracted from 'end' field.
        This date is converted to a perceval format using a float value.

        :param item: item generated by the backend

        :returns: a UNIX timestamp
        """
        if 'end' in item:
            # events updated field
            updated = item['end']
        elif 'date_joined_program' in item:
            # users updated field that always appear
            updated = item['date_joined_program']
        elif 'report_date' in item:
            # activities updated field
            updated = item['report_date']
        else:
            raise ValueError("Can't find updated field for item " + str(item))

        return float(str_to_datetime(updated).timestamp())

    @staticmethod
    def metadata_category(item):
        """Extracts the category from a ReMo item.

        This backend generates items types 'event', 'activity'
        or 'user'. To guess the type of item, the code will look
        for unique fields.
        """
        if 'estimated_attendance' in item:
            category = CATEGORY_EVENT
        elif 'activity' in item:
            category = CATEGORY_ACTIVITY
        elif 'first_name' in item:
            category = CATEGORY_USER
        else:
            raise TypeError("Could not define the category of item " + str(item))

        return category

    def _init_client(self, from_archive=False):
        """Init client"""

        return ReMoClient(self.url, self.archive, from_archive, self.ssl_verify)


class ReMoClient(HttpClient):
    """ReMo API client.

    This class implements a simple client to retrieve events from
    projects in a ReMo site.

    :param url: URL of ReMo (sample https://reps.mozilla.org)
    :param archive: an archive to store/read fetched data
    :param from_archive: it tells whether to write/read the archive
    :param ssl_verify: enable/disable SSL verification

    :raises HTTPError: when an error occurs doing the request
    """

    FIRST_PAGE = 1  # Initial page in ReMo API
    ITEMS_PER_PAGE = 20  # Items per page in ReMo API
    API_PATH = '/api/remo/v1'

    def __init__(self, url, archive=None, from_archive=False, ssl_verify=True):
        super().__init__(url, archive=archive, from_archive=from_archive, ssl_verify=ssl_verify)
        self.api_activities_url = urijoin(self.base_url, ReMoClient.API_PATH + '/activities/')
        self.api_activities_url += '/'  # API needs a final /
        self.api_events_url = urijoin(self.base_url, ReMoClient.API_PATH + '/events/')
        self.api_events_url += '/'  # API needs a final /
        self.api_users_url = urijoin(self.base_url, ReMoClient.API_PATH + '/users/')
        self.api_users_url += '/'  # API needs a final /

    def get_items(self, category=CATEGORY_EVENT, offset=REMO_DEFAULT_OFFSET):
        """Retrieve all items for category using pagination """

        more = True  # There are more items to be processed
        next_uri = None  # URI for the next items page query
        page = ReMoClient.FIRST_PAGE
        page += int(offset / ReMoClient.ITEMS_PER_PAGE)

        if category == CATEGORY_EVENT:
            api = self.api_events_url
        elif category == CATEGORY_ACTIVITY:
            api = self.api_activities_url
        elif category == CATEGORY_USER:
            api = self.api_users_url
        else:
            raise ValueError(category + ' not supported in ReMo')

        while more:
            params = {
                "page": page,
                "orderby": "ASC"
            }

            logger.debug("ReMo client calls APIv2: %s params: %s",
                         api, str(params))

            raw_items = self.fetch(api, payload=params)
            yield raw_items

            items_data = json.loads(raw_items)
            next_uri = items_data['next']

            if not next_uri:
                more = False
            else:
                # https://reps.mozilla.org/remo/api/remo/v1/events/?orderby=ASC&page=269
                parsed_uri = urllib.parse.urlparse(next_uri)
                parsed_params = urllib.parse.parse_qs(parsed_uri.query)
                page = parsed_params['page'][0]

    def fetch(self, url, payload=None):
        """Return the textual content associated to the Response object"""

        response = super().fetch(url, payload)

        return response.text


class ReMoCommand(BackendCommand):
    """Class to run ReMo backend from the command line."""

    BACKEND = ReMo

    @classmethod
    def setup_cmd_parser(cls):
        """Returns the ReMo argument parser."""

        parser = BackendCommandArgumentParser(cls.BACKEND,
                                              offset=True,
                                              archive=True,
                                              ssl_verify=True)
        # Required arguments
        parser.parser.add_argument('url', nargs='?',
                                   default="https://reps.mozilla.org",
                                   help="ReMo URL (default: https://reps.mozilla.org)")

        return parser



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Santiago Dueñas <sduenas@bitergia.com>
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import unittest

import httpretty

from perceval.backend import BackendCommandArgumentParser
from perceval.backends.mozilla.mozillaclub import (MozillaClub,
                                                   MozillaClubCommand,
                                                   MozillaClubClient,
                                                   MozillaClubParser)
from base import TestCaseBackendArchive


MozillaClub_FEED_URL = 'http://example.com/feed'

requests_http = []


def read_file(filename, mode='r'):
    with open(filename, mode) as f:
        content = f.read()
    return content


def configure_http_server():
    bodies_events_job = read_file('data/mozillaclub/feed.json')

    http_requests = []

    def request_callback(method, uri, headers):
        last_request = httpretty.last_request()

        if uri.startswith(MozillaClub_FEED_URL):
            body = bodies_events_job
        else:
            body = ''

        requests_http.append(httpretty.last_request())

        http_requests.append(last_request)

        return (200, headers, body)

    httpretty.register_uri(httpretty.GET,
                           MozillaClub_FEED_URL,
                           responses=[
                               httpretty.Response(body=request_callback)
                               for _ in range(2)
                           ])

    return http_requests


class TestMozillaClubBackend(unittest.TestCase):
    """MozillaClub backend tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        mozillaclub = MozillaClub(MozillaClub_FEED_URL, tag='test')

        self.assertEqual(mozillaclub.url, MozillaClub_FEED_URL)
        self.assertEqual(mozillaclub.origin, MozillaClub_FEED_URL)
        self.assertEqual(mozillaclub.tag, 'test')
        self.assertIsNone(mozillaclub.client)
        self.assertTrue(mozillaclub.ssl_verify)

        # When tag is empty or None it will be set to
        # the value in url
        mozillaclub = MozillaClub(MozillaClub_FEED_URL, ssl_verify=False)
        self.assertEqual(mozillaclub.url, MozillaClub_FEED_URL)
        self.assertEqual(mozillaclub.origin, MozillaClub_FEED_URL)
        self.assertEqual(mozillaclub.tag, MozillaClub_FEED_URL)
        self.assertFalse(mozillaclub.ssl_verify)

        mozillaclub = MozillaClub(MozillaClub_FEED_URL, tag='')
        self.assertEqual(mozillaclub.url, MozillaClub_FEED_URL)
        self.assertEqual(mozillaclub.origin, MozillaClub_FEED_URL)
        self.assertEqual(mozillaclub.tag, MozillaClub_FEED_URL)
        self.assertTrue(mozillaclub.ssl_verify)

    def test_has_archiving(self):
        """Test if it returns True when has_archiving is called"""

        self.assertEqual(MozillaClub.has_archiving(), True)

    def test_has_resuming(self):
        """Test if it returns False when has_resuming is called"""

        self.assertEqual(MozillaClub.has_resuming(), False)

    @httpretty.activate
    def test_fetch(self):
        """Test whether a list of events is returned"""

        http_requests = configure_http_server()

        # Test fetch events from feed
        mozillaclub = MozillaClub(MozillaClub_FEED_URL)
        events = [event for event in mozillaclub.fetch()]
        self.assertEqual(len(events), 92)
        self.assertEqual(len(http_requests), 1)

        # Test metadata
        expected = [('a60e67f5094f325f1cc826159749eaecbd177fc9', 1481643844.821,
                     'Rio Mozilla Club'),
                    ('2e6eaf1f174600d20233e5051e85d730952a8a31', 1481643844.821,
                     'Firefox club uog-skt'),
                    ('da8820deeb01616d22d6851272e8db915efc351b', 1481643844.821,
                     'Mozilla HETEC Club')]

        for x in range(len(expected)):
            event = events[x]
            self.assertEqual(event['origin'], 'http://example.com/feed')
            self.assertEqual(event['uuid'], expected[x][0])
            self.assertEqual(event['updated_on'], expected[x][1])
            self.assertEqual(event['category'], 'event')
            self.assertEqual(event['tag'], 'http://example.com/feed')
            self.assertEqual(event['data']['Club Name'], expected[x][2])

    @httpretty.activate
    def test_empty_cells(self):
        """
        Test whether the empty fields cells that are not included in the json
        feed, are filled with None in the generated event.
        """

        http_requests = configure_http_server()

        # https://docs.google.com/spreadsheets/d/1QHl2bjBhMslyFzR5XXPzMLdzzx7oeSKTbgR5PM8qp64/pubhtml
        # Club Link is always empty so it is not included in
        # https://spreadsheets.google.com/feeds/cells/1QHl2bjBhMslyFzR5XXPzMLdzzx7oeSKTbgR5PM8qp64/ohaibtm/public/values?alt=json
        # Check that the field 'Club Link' exist in the items with None value

        mozillaclub = MozillaClub(MozillaClub_FEED_URL)
        events = [event for event in mozillaclub.fetch()]
        self.assertEqual(len(events), 92)
        for event in events:
            self.assertEqual('Club Link' in event['data'].keys(), True)
            self.assertEqual(event['data']['Club Link'], None)

    @httpretty.activate
    def test_fetch_empty(self):
        """Test whether it works when no events are fetched"""

        body = """
            {
                "encoding": "UTF-8",
                "feed": {
                "event": []
                }
            }
        """
        httpretty.register_uri(httpretty.GET,
                               MozillaClub_FEED_URL,
                               body=body, status=200)

        mozillaclub = MozillaClub(MozillaClub_FEED_URL)
        events = [event for event in mozillaclub.fetch()]

        self.assertEqual(len(events), 0)

    @httpretty.activate
    def test_search_fields(self):
        """Test whether the search_fields is properly set"""

        configure_http_server()

        mozillaclub = MozillaClub(MozillaClub_FEED_URL)
        events = [event for event in mozillaclub.fetch()]

        event = events[0]
        self.assertEqual(mozillaclub.metadata_id(event['data']), event['search_fields']['item_id'])
        self.assertEqual(event['data']['Club Name'], 'Rio Mozilla Club')
        self.assertEqual(event['data']['Club Name'], event['search_fields']['club_name'])

        event = events[1]
        self.assertEqual(mozillaclub.metadata_id(event['data']), event['search_fields']['item_id'])
        self.assertEqual(event['data']['Club Name'], 'Firefox club uog-skt')
        self.assertEqual(event['data']['Club Name'], event['search_fields']['club_name'])

        event = events[2]
        self.assertEqual(mozillaclub.metadata_id(event['data']), event['search_fields']['item_id'])
        self.assertEqual(event['data']['Club Name'], 'Mozilla HETEC Club')
        self.assertEqual(event['data']['Club Name'], event['search_fields']['club_name'])


class TestMozillaClubBackendArchive(TestCaseBackendArchive):
    """MozillaClub backend tests using an archive"""

    def setUp(self):
        super().setUp()
        self.backend_write_archive = MozillaClub(MozillaClub_FEED_URL, archive=self.archive)
        self.backend_read_archive = MozillaClub(MozillaClub_FEED_URL, archive=self.archive)

    @httpretty.activate
    def test_fetch_from_archive(self):
        """Test whether a list of events is returned from archive"""

        configure_http_server()
        self._test_fetch_from_archive()

    @httpretty.activate
    def test_fetch_empty_from_archive(self):
        """Test whether it works when no events are fetched from archive"""

        body = """
                {
                    "encoding": "UTF-8",
                    "feed": {
                    "event": []
                    }
                }
            """
        httpretty.register_uri(httpretty.GET,
                               MozillaClub_FEED_URL,
                               body=body, status=200)

        self._test_fetch_from_archive()


class TestMozillaClubCommand(unittest.TestCase):
    """Tests for MozillaClubCommand class"""

    def test_backend_class(self):
        """Test if the backend class is MozillaClub"""

        self.assertIs(MozillaClubCommand.BACKEND, MozillaClub)

    def test_setup_cmd_parser(self):
        """Test if it parser object is correctly initialized"""

        parser = MozillaClubCommand.setup_cmd_parser()
        self.assertIsInstance(parser, BackendCommandArgumentParser)
        self.assertEqual(parser._backend, MozillaClub)

        args = [MozillaClub_FEED_URL,
                '--tag', 'test',
                '--no-archive']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.url, MozillaClub_FEED_URL)
        self.assertEqual(parsed_args.tag, 'test')
        self.assertEqual(parsed_args.no_archive, True)
        self.assertTrue(parsed_args.ssl_verify)

        args = [MozillaClub_FEED_URL, '--no-ssl-verify']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.url, MozillaClub_FEED_URL)
        self.assertFalse(parsed_args.ssl_verify)


class TestMozillaClubClient(unittest.TestCase):
    """MozillaClub API client tests

    These tests not check the body of the response, only if the call
    was well formed and if a response was obtained. Due to this, take
    into account that the body returned on each request might not
    match with the parameters from the request.
    """

    def test_init(self):
        """Test initialization"""

        mozillaclub = MozillaClubClient(MozillaClub_FEED_URL)

        self.assertEqual(mozillaclub.base_url, MozillaClub_FEED_URL)
        self.assertIsNone(mozillaclub.archive)
        self.assertFalse(mozillaclub.from_archive)
        self.assertTrue(mozillaclub.ssl_verify)

        mozillaclub = MozillaClubClient(MozillaClub_FEED_URL, ssl_verify=False)

        self.assertEqual(mozillaclub.base_url, MozillaClub_FEED_URL)
        self.assertIsNone(mozillaclub.archive)
        self.assertFalse(mozillaclub.from_archive)
        self.assertFalse(mozillaclub.ssl_verify)

    @httpretty.activate
    def test_get_events(self):
        """Test get_events API call"""

        # Set up a mock HTTP server
        body = read_file('data/mozillaclub/feed.json')
        httpretty.register_uri(httpretty.GET,
                               MozillaClub_FEED_URL,
                               body=body, status=200)

        client = MozillaClubClient(MozillaClub_FEED_URL)
        response = client.get_cells()

        self.assertEqual(response, body)


class TestMozillaClubParser(unittest.TestCase):
    """MozillaClub parser tests"""

    def test_parser(self):
        """Test if it parsers a JSON feed stream"""

        with open("data/mozillaclub/feed.json", 'r') as f:
            parser = MozillaClubParser(f.read())
            events = [event for event in parser.parse()]

        self.assertEqual(len(events), 92)

        # Checking some random data
        self.assertEqual(events[10]['City'], 'Cape Town')
        self.assertEqual(events[45]['Links to Photos (Optional)'], None)
        self.assertEqual(events[53]['updated'], '2016-12-13T15:44:04.821Z')


if __name__ == "__main__":
    unittest.main(warnings='ignore')
